// exdump - 0.2 - By PolarRoot [pi9@hotmail.com]
//	dumpicmp.c - ICMP Dump Routines

#include "main.h"

// process_icmp_packet() Function
void process_icmp_packet (struct ICMP_HEADER packet, int size) {
	char type[32];
	char code[32];
	struct protoent prot;

	if (ignore_icmp_packet(packet)) return;

	icmp_type(packet, type);
	icmp_code(packet, code);
	prot=*getprotobynumber(packet.ip.ip_p);

	if (OPT_VERBOSE_MODE!=1) {
		fprintf(hOUTPUT_FILE, "ICMP: From: %s   To: %s   Size: %d  [%s: %s]\n", 
			inet_ntoa(packet.ip.ip_src), inet_ntoa(packet.ip.ip_dst),
			size, type, code);
		fflush(hOUTPUT_FILE);
	}
	else {
		fprintf(hOUTPUT_FILE, "ICMP Packet Received [Size: %i]\n", size);
		fprintf(hOUTPUT_FILE, "IP Header:\n");
		fprintf(hOUTPUT_FILE, "\tHeader Length: %d\n", packet.ip.ip_hl);
		fprintf(hOUTPUT_FILE, "\tVersion: %d\n", packet.ip.ip_v);
		fprintf(hOUTPUT_FILE, "\tType of Service: %d\n", packet.ip.ip_tos);
		fprintf(hOUTPUT_FILE, "\tTotal Length: %d\n", packet.ip.ip_len);
		fprintf(hOUTPUT_FILE, "\tIdentification: %d\n", packet.ip.ip_id);
		fprintf(hOUTPUT_FILE, "\tFragment Offset Field: %d\n", packet.ip.ip_off);
		fprintf(hOUTPUT_FILE, "\tTime to Live: %d\n", packet.ip.ip_ttl);
		fprintf(hOUTPUT_FILE, "\tProtocol: %d (%s)\n", packet.ip.ip_p, prot.p_name);
		fprintf(hOUTPUT_FILE, "\tChecksum: %d\n", packet.ip.ip_sum);
		fprintf(hOUTPUT_FILE, "\tSource Address: %s\n", inet_ntoa(packet.ip.ip_src));
		fprintf(hOUTPUT_FILE, "\tDestination Address: %s\n", inet_ntoa(packet.ip.ip_dst));
		fprintf(hOUTPUT_FILE, "\n");
		fprintf(hOUTPUT_FILE, "ICMP Header:\n");
		fprintf(hOUTPUT_FILE, "\tType: %d (%s)\n", packet.icmp.type, type);
		fprintf(hOUTPUT_FILE, "\tCode: %d (%s)\n", packet.icmp.code, code);
		fprintf(hOUTPUT_FILE, "\tChecksum: %ld\n", packet.icmp.checksum);
		fprintf(hOUTPUT_FILE, "\tEcho ID: %d\n", packet.icmp.un.echo.id);
		fprintf(hOUTPUT_FILE, "\tEcho Sequence: %d\n", packet.icmp.un.echo.sequence);
		fprintf(hOUTPUT_FILE, "\tGateway: %ld\n", packet.icmp.un.gateway);
		fprintf(hOUTPUT_FILE, "\tFragment MTU: %d\n", packet.icmp.un.frag.mtu);
		if (OPT_NO_DATA==0) {
			fprintf(hOUTPUT_FILE, "\nData:\n");
			fprintf(hOUTPUT_FILE, packet.space);
			fprintf(hOUTPUT_FILE, "\n");
		}
		fprintf(hOUTPUT_FILE, "\n\n");
		fflush(hOUTPUT_FILE);
	}
}

// icmp_type() Function
void icmp_type (struct ICMP_HEADER packet, char type[32]) {
	if (OPT_VERBOSE_MODE!=1) {
		if (packet.icmp.type==ICMP_ECHOREPLY) strcpy(type, "   ECHO");
		if (packet.icmp.type==ICMP_DEST_UNREACH) strcpy(type, "UNREACH");
		if (packet.icmp.type==ICMP_SOURCE_QUENCH) strcpy(type, " QUENCH");
		if (packet.icmp.type==ICMP_REDIRECT) strcpy(type, "  REDIR");
		if (packet.icmp.type==ICMP_ECHO) strcpy(type, "ECHOREQ");
		if (packet.icmp.type==ICMP_TIME_EXCEEDED) strcpy(type, " TIMEEX");
		if (packet.icmp.type==ICMP_PARAMETERPROB) strcpy(type, "   PROB");
		if (packet.icmp.type==ICMP_TIMESTAMP) strcpy(type, " TIMREQ");
		if (packet.icmp.type==ICMP_TIMESTAMPREPLY) strcpy(type, " TIMREP");
		if (packet.icmp.type==ICMP_INFO_REQUEST) strcpy(type, " INFREQ");
		if (packet.icmp.type==ICMP_INFO_REPLY) strcpy(type, " INFREP");
		if (packet.icmp.type==ICMP_ADDRESS) strcpy(type, "ADDRREQ");
		if (packet.icmp.type==ICMP_ADDRESSREPLY) strcpy(type, "ADDRREP");
	} else {
		if (packet.icmp.type==ICMP_ECHOREPLY) strcpy(type, "Echo Reply");
		if (packet.icmp.type==ICMP_DEST_UNREACH) strcpy(type, "Destination Unreachable");
		if (packet.icmp.type==ICMP_SOURCE_QUENCH) strcpy(type, "Source Quench");
		if (packet.icmp.type==ICMP_REDIRECT) strcpy(type, "Redirect");
		if (packet.icmp.type==ICMP_ECHO) strcpy(type, "Echo Request");
		if (packet.icmp.type==ICMP_TIME_EXCEEDED) strcpy(type, "Time Exceeded");
		if (packet.icmp.type==ICMP_PARAMETERPROB) strcpy(type, "Parameter Problem");
		if (packet.icmp.type==ICMP_TIMESTAMP) strcpy(type, "Timestamp Request");
		if (packet.icmp.type==ICMP_TIMESTAMPREPLY) strcpy(type, "Timestamp Reply");
		if (packet.icmp.type==ICMP_INFO_REQUEST) strcpy(type, "Information Request");
		if (packet.icmp.type==ICMP_INFO_REPLY) strcpy(type, "Information Reply");
		if (packet.icmp.type==ICMP_ADDRESS) strcpy(type, "Address Mask Request");
		if (packet.icmp.type==ICMP_ADDRESSREPLY) strcpy(type, "Address Mask Reply");
	}
	return;
}

// icmp_code() Function
void icmp_code (struct ICMP_HEADER packet, char code[32]) {
	if (OPT_VERBOSE_MODE!=1) {
		if (packet.icmp.type==ICMP_ECHOREPLY) strcpy(code, "none");
		if (packet.icmp.type==ICMP_DEST_UNREACH) {
			if (packet.icmp.code==ICMP_NET_UNREACH) strcpy(code, "netr");
			if (packet.icmp.code==ICMP_HOST_UNREACH) strcpy(code, "hostr");
			if (packet.icmp.code==ICMP_PROT_UNREACH) strcpy(code, "protr");
			if (packet.icmp.code==ICMP_PORT_UNREACH) strcpy(code, "portr");
			if (packet.icmp.code==ICMP_FRAG_NEEDED) strcpy(code, "frag");
			if (packet.icmp.code==ICMP_SR_FAILED) strcpy(code, "route");
			if (packet.icmp.code==ICMP_NET_UNKNOWN) strcpy(code, "netn");
			if (packet.icmp.code==ICMP_HOST_UNKNOWN) strcpy(code, "hostn");
			if (packet.icmp.code==ICMP_HOST_ISOLATED) strcpy(code, "hosti");
			if (packet.icmp.code==ICMP_NET_ANO) strcpy(code, "netp");
			if (packet.icmp.code==ICMP_HOST_ANO) strcpy(code, "hostp");
			if (packet.icmp.code==ICMP_NET_UNR_TOS) strcpy(code, "ntos");
			if (packet.icmp.code==ICMP_HOST_UNR_TOS) strcpy(code, "htos");
			if (packet.icmp.code==ICMP_PKT_FILTERED) strcpy(code, "filt");
			if (packet.icmp.code==ICMP_PREC_VIOLATION) strcpy(code, "precv");
			if (packet.icmp.code==ICMP_PREC_CUTOFF) strcpy(code, "precc");
		}
		if (packet.icmp.type==ICMP_SOURCE_QUENCH) strcpy(code, "none");
		if (packet.icmp.type==ICMP_REDIRECT) {
			if (packet.icmp.code==ICMP_REDIR_NET) strcpy(code, "net");
			if (packet.icmp.code==ICMP_REDIR_HOST) strcpy(code, "host");
			if (packet.icmp.code==ICMP_REDIR_NETTOS) strcpy(code, "ntos");
			if (packet.icmp.code==ICMP_REDIR_HOSTTOS) strcpy(code, "htos");
		}
		if (packet.icmp.type==ICMP_ECHO) strcpy(code, "none");
		if (packet.icmp.type==ICMP_TIME_EXCEEDED) {
			if (packet.icmp.code==ICMP_EXC_TTL) strcpy(code, "ttl");
			if (packet.icmp.code==ICMP_EXC_FRAGTIME) strcpy(code, "frag");
		}
		if (packet.icmp.type==ICMP_PARAMETERPROB) {
			if (packet.icmp.code==0) strcpy(code, "headr");
			if (packet.icmp.code==1) strcpy(code, "reqop");
		}
		if (packet.icmp.type==ICMP_TIMESTAMP) strcpy(code, "none");
		if (packet.icmp.type==ICMP_TIMESTAMPREPLY) strcpy(code, "none");
		if (packet.icmp.type==ICMP_INFO_REQUEST) strcpy(code, "none");
		if (packet.icmp.type==ICMP_INFO_REPLY) strcpy(code, "none");
		if (packet.icmp.type==ICMP_ADDRESS) strcpy(code, "none");
		if (packet.icmp.type==ICMP_ADDRESSREPLY) strcpy(code, "none");
	} else {
		if (packet.icmp.type==ICMP_ECHOREPLY) strcpy(code, "none");
		if (packet.icmp.type==ICMP_DEST_UNREACH) {
			if (packet.icmp.code==ICMP_NET_UNREACH) strcpy(code, "network");
			if (packet.icmp.code==ICMP_HOST_UNREACH) strcpy(code, "host");
			if (packet.icmp.code==ICMP_PROT_UNREACH) strcpy(code, "protocol");
			if (packet.icmp.code==ICMP_PORT_UNREACH) strcpy(code, "port");
			if (packet.icmp.code==ICMP_FRAG_NEEDED) strcpy(code, "fragment needed");
			if (packet.icmp.code==ICMP_SR_FAILED) strcpy(code, "source route failed");
			if (packet.icmp.code==ICMP_NET_UNKNOWN) strcpy(code, "network unknown");
			if (packet.icmp.code==ICMP_HOST_UNKNOWN) strcpy(code, "host unknown");
			if (packet.icmp.code==ICMP_HOST_ISOLATED) strcpy(code, "host isolated");
			if (packet.icmp.code==ICMP_NET_ANO) strcpy(code, "network prohibited");
			if (packet.icmp.code==ICMP_HOST_ANO) strcpy(code, "host prohibited");
			if (packet.icmp.code==ICMP_NET_UNR_TOS) strcpy(code, "network type-of-service");
			if (packet.icmp.code==ICMP_HOST_UNR_TOS) strcpy(code, "host type-of-service");
			if (packet.icmp.code==ICMP_PKT_FILTERED) strcpy(code, "packet filtered");
			if (packet.icmp.code==ICMP_PREC_VIOLATION) strcpy(code, "precedence violation");
			if (packet.icmp.code==ICMP_PREC_CUTOFF) strcpy(code, "precedence cut-off");
		}
		if (packet.icmp.type==ICMP_SOURCE_QUENCH) strcpy(code, "none");
		if (packet.icmp.type==ICMP_REDIRECT) {
			if (packet.icmp.code==ICMP_REDIR_NET) strcpy(code, "network");
			if (packet.icmp.code==ICMP_REDIR_HOST) strcpy(code, "host");
			if (packet.icmp.code==ICMP_REDIR_NETTOS) strcpy(code, "network type-of-service");
			if (packet.icmp.code==ICMP_REDIR_HOSTTOS) strcpy(code, "host type-of-service");
		}
		if (packet.icmp.type==ICMP_ECHO) strcpy(code, "none");
		if (packet.icmp.type==ICMP_TIME_EXCEEDED) {
			if (packet.icmp.code==ICMP_EXC_TTL) strcpy(code, "time-to-live");
			if (packet.icmp.code==ICMP_EXC_FRAGTIME) strcpy(code, "fragment time");
		}
		if (packet.icmp.type==ICMP_PARAMETERPROB) {
			if (packet.icmp.code==0) strcpy(code, "ip header");
			if (packet.icmp.code==1) strcpy(code, "required option");
		}
		if (packet.icmp.type==ICMP_TIMESTAMP) strcpy(code, "none");
		if (packet.icmp.type==ICMP_TIMESTAMPREPLY) strcpy(code, "none");
		if (packet.icmp.type==ICMP_INFO_REQUEST) strcpy(code, "none");
		if (packet.icmp.type==ICMP_INFO_REPLY) strcpy(code, "none");
		if (packet.icmp.type==ICMP_ADDRESS) strcpy(code, "none");
		if (packet.icmp.type==ICMP_ADDRESSREPLY) strcpy(code, "none");
	}
	return;
}

// ignore_icmp_packet() Function
int ignore_icmp_packet (struct ICMP_HEADER packet) {
	if (OPT_WATCH_ALL==1) return 0;
// DEBUG
//	if (OPT_WATCH_LOCAL==1 & inet_ntoa(packet.ip.ip_dst)=="127.0.0.1" | inet_ntoa(packet.ip.ip_dst)=="localhost" | inet_ntoa(packet.ip.ip_dst)==LOCALHOST_NAME | inet_ntoa(packet.ip.ip_dst)==inet_ntoa(*(long *)(host->h_addr))) return 0;
//	if (OPT_WATCH_OUTSIDE==1 & inet_ntoa(packet.ip.ip_dst)!="127.0.0.1") return 0;
	return 1;
}

